<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2007 The Cacti Group                                      |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/



$dir = dirname(__FILE__);
chdir($dir);

if (strpos($dir, 'update') !== false) {
	chdir('../../');
}

/* Start Initialization Section */
if (file_exists("./include/global.php")) {
	include("./include/global.php");
} else {
	include("./include/config.php");
}
include_once($config["base_path"] . "/plugins/update/functions.php");

ini_set("max_execution_time", 30);


/* process calling arguments */
$parms = $_SERVER["argv"];
array_shift($parms);


$debug = FALSE;
$forcerun = FALSE;

foreach($parms as $parameter) {
	@list($arg, $value) = @explode("=", $parameter);

	switch ($arg) {
	case "-d":
		$debug = TRUE;
		break;
	case "-f":
		$forcerun = TRUE;
		break;
	case "--help":
	case "-h":
		display_help();
		exit;
	default:
		print "ERROR: Invalid Parameter " . $parameter . "\n\n";
		display_help();
		exit;
	}
}

update_info_table ();

$last_check = read_config_option("plugin_update_last_check");
if ($last_check < 1)
	$last_check = 0;

$current_time = time();

$interval = read_config_option("plugin_update_check_time");
if ($interval == '' || $interval > 4 || $interval < 0)
	$interval = 0;

if ($interval == 0) {
	if ($debug)
		print "Your Plugin Scan Interval is either not set or is set to Never\n";
	exit;
}

print "interval = $interval\n";
$s = array(86400, 172800, 604800, 2678400);
$seconds = $s[$interval - 1];
$last_check = read_config_option("plugin_update_last_check");
if ($last_check < 1)
	$last_check = 0;
$current_time = time();
if ($current_time < $last_check + $seconds) {
	if ($forcerun == false) {
		if ($debug) {
			print "You must wait 24 hours between checks to prevent hammering the servers.\n";
			print "Last Scan ran at : " . date("F j, Y, g:i:s a", $last_check) . "\n";
			print "Next Scan at     : " . date("F j, Y, g:i:s a", $last_check + $seconds) . "\n";	
		}
		exit;
	} else {
		print "************************************************************************\n";
		print "* You are currently forcing the checker to run, even though it has not *\n";
		print "* been 24 hours since the last polling.  This is currently allowed for *\n";
		print "* debugging purposes, but if abused, you will be blocked from checking *\n";
		print "* updates on the server side PERMANENTLY!!!                            *\n";
		print "************************************************************************\n";
		print "Last Scan ran at : " . date("F j, Y, g:i:s a", $last_check) . "\n";
	}
}

if ($debug)
	echo "Starting Checker....\n";
cacti_log("Plugin Update Check is now processing", true, "POLLER");

db_execute("REPLACE INTO settings (name, value) VALUES ('plugin_update_last_check', '$current_time')");

update_fetch_new_versions ();



/*	display_help - displays the usage of the function */
function display_help () {
	$version = update_version ();
	$version = $version['version'];
	print "-----------------------------------------------------------------------\n";
	print "Plugin Update Checker Version $version\n Copyright 2006 - Jimmy Conner\n\n";
	print "usage: check.php [-d] [-f] [-h] [--help]\n\n";
	print "-f            - Force the checking for Plugin Updates\n";
	print "                (must wait 1 minute between checks)\n";
	print "-d            - Display verbose output during execution\n";
	print "-h --help     - display this help message\n";
	print "-----------------------------------------------------------------------\n";
	exit;
}

?>