<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2007 The Cacti Group                                      |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/


function update_info_table () {
	global $plugins, $plugin_architecture;

	$plugins2 = $plugins;
	$plugins2[] = 'cacti';
	if (isset($plugin_architecture['version'])) {
		$plugins2[] = 'cacti_plugin_arch';
	}
	$cinfo = update_get_plugin_info ();
	$info = update_get_cached_plugin_info();
	
	foreach ($plugins2 as $plugin) {
		if (!isset($info[$plugin]) && isset($cinfo[$plugin])) {
			db_execute("INSERT INTO plugin_update_info (plugin, name, version, author, homepage, email) VALUES (" .
			"'$plugin' ," .
			"'" . (isset($cinfo[$plugin]['longname']) ? sql_sanitize($cinfo[$plugin]['longname']) : "") . "' ," .
			"'" . sql_sanitize($cinfo[$plugin]['version']) . "' ," .	
			"'" . (isset($cinfo[$plugin]['author']) ? sql_sanitize($cinfo[$plugin]['author']) : "") . "' ," .
			"'" . (isset($cinfo[$plugin]['homepage']) ? sql_sanitize($cinfo[$plugin]['homepage']) : "") . "' ," .
			"'" . (isset($cinfo[$plugin]['email']) ? sql_sanitize($cinfo[$plugin]['email']) : "") . "'" .
			"'" . (isset($cinfo[$plugin]['changes']) ? sql_sanitize($cinfo[$plugin]['changes']) : "") . "'" .
			")");
		}
	}
}

function update_fetch_new_versions () {
	global $plugins, $debug, $plugin_architecture;

	$plugins2 = $plugins;
	$plugins2[] = 'cacti';
	if (isset($plugin_architecture['version'])) {
		$plugins2[] = 'cacti_plugin_arch';
	}

	$cinfo = update_get_plugin_info ();

	$info = update_get_cached_plugin_info ();

	foreach ($plugins2 as $plugin) {
		if ($debug)
			print "Checking Version information for '$plugin'\n";
		if (isset($cinfo[$plugin]['url']) && $cinfo[$plugin]['url'] != '') {
			$nv = update_fetch_version($cinfo[$plugin]['url'], $plugin);
			if ($nv != '') {
				if (update_check_if_newer($cinfo[$plugin]['version'], $nv)) {
					if ($debug)
						print "   Online version is newer!\n";
					if ($nv != $info[$plugin]['version']) {
						$changes = update_fetch_changes($cinfo[$plugin]['url'], $plugin);
						$changes = str_replace(array("'",'"', "\\"),  "", $changes);
						db_execute("UPDATE plugin_update_info set changes = '$changes' where plugin = '$plugin'");
					} else {
						if ($debug)
							print "   Already have the latest changes.\n";
					}
				}
				$nv = str_replace(array("'",'"', ';', "\\", '/'),  "", $nv);
				db_execute("UPDATE plugin_update_info set version = '$nv' where plugin = '$plugin'");
			}
		} else {
			if ($debug)
				print "  Plugin does not contain a URL to check!\n";
		}
	}
}

function update_check_if_newer ($current, $new) {
	if ($new > $current) return true;
	return false;
}

// This was the first way of doing it, but it didn't allow me to set timeouts, which was bad if we couldn't DNS
function update_fetch_version2($url, $plugin) {
	global $debug;

	$url = $url . "?plugin=$plugin&fetch=version";
	if ($debug)
		print "   Using URL : " . $url . "\n";

	$f = fopen($url, 'r');
	if ($f) {
		if ($debug)
			print "   Connected to Host successfully\n";
		stream_set_timeout($f, 2);
		$data = fgets($f, 512);
		fclose($f);
		if ($debug)
			print "   Newest version is : $data\n";
		return $data;
	} else {
		if ($debug)
			print "   Could not connect to host\n";
	}
	return '';
}

function update_fetch_version ($url, $plugin) {
	global $debug;

	$url = $url . "?plugin=$plugin&fetch=version";

	$url2 = str_replace("http://","",$url);
	$url2 = substr($url2, strpos($url2, '/'));


	$host = str_replace("http://","",$url);
	$host = substr($host, 0, strpos($host, '/'));

	$version = update_version ();
	$version = $version['version'];

	if ($debug) {
		print "   Using Host : " . $host . "\n";
		print "   Using URL  : " . $url2 . "\n";
	}

	$fp = @fsockopen($host, 80, $errno, $errstr, 3);
	if (!$fp) {
		if ($debug)
			print "   Error: $errstr ($errno)\n";
	} else {
		if ($debug)
			print "   Connected to Host successfully\n";
		$out  = "GET $url2 HTTP/1.0\r\n";
		$out .= "User-Agent: Cacti Version Checker v$version\r\n";
		$out .= "Host: $host\r\n";
		$out .= "Connection: Close\r\n\r\n";

		@fwrite($fp, $out);

		$statusline = @fgets($fp, 1024);
		// check here for non 200!
               
		$in_data = 0;
		$data = "";
		if (is_resource($fp)) {
			while(!@feof($fp)) {
				$data .= @fgets($fp,1024);     
				if( $in_data==0 && preg_match('/^\s*$/',$data)) {
					// From now on, save what comes in
					$in_data = 1;
					$data = '';
				}
				if( $in_data==0) $data = '';
			}
		} 

		@fclose($fp);
		if ($debug)
			print "   Newest version is : $data\n";
		return $data;
	}
	return '';
}

function update_fetch_changes ($url, $plugin) {
	global $debug;

	$url = $url . "?plugin=$plugin&fetch=changes";

	$url2 = str_replace("http://","",$url);
	$url2 = substr($url2, strpos($url2, '/'));


	$host = str_replace("http://","",$url);
	$host = substr($host, 0, strpos($host, '/'));

	$version = update_version ();
	$version = $version['version'];

	if ($debug) {
		print "Checking Changes information for '$plugin'\n";
		print "   Using Host : " . $host . "\n";
		print "   Using URL  : " . $url2 . "\n";
	}

	$fp = @fsockopen($host, 80, $errno, $errstr, 3);
	if (!$fp) {
		if ($debug)
			print "   Error: $errstr ($errno)\n";
	} else {
		if ($debug)
			print "   Connected to Host successfully\n";
		$out  = "GET $url2 HTTP/1.0\r\n";
		$out .= "User-Agent: Cacti Version Checker v$version\r\n";
		$out .= "Host: $host\r\n";
		$out .= "Connection: Close\r\n\r\n";

		@fwrite($fp, $out);

		$statusline = @fgets($fp, 1024);
		// check here for non 200!
               
		$in_data = 0;
		$data = "";
		if (is_resource($fp)) {
			while(!@feof($fp)) {
				$data .= @fgets($fp,1024);     
				if( $in_data==0 && preg_match('/^\s*$/',$data)) {
					// From now on, save what comes in
					$in_data = 1;
					$data = '';
				}
				if( $in_data==0) $data = '';
			}
		} 

		@fclose($fp);
		if ($debug)
			print "   Latest Changes : $data\n";
		return $data;
	}
	return '';
}

function update_get_plugin_info () {
	global $plugins, $plugin_architecture, $debug;
	$cinfo = array();
	if ($debug) 
		print "Retrieving current plugin information from internal functions.\n";
	$plugins2 = $plugins;
	$plugins2[] = 'cacti';
	if (isset($plugin_architecture['version'])) {
		$plugins2[] = 'cacti_plugin_arch';
	}

	foreach ($plugins2 as $plugin) {
		$func = $plugin . "_version";
		if ($debug) 
			print "   Polling plugin '$plugin' using function '$func'.\n";
		if (function_exists($func)) {
			$cinfo[$plugin] = $func();
			if ($debug)
				print "      Current version is : " . $cinfo[$plugin]['version'] . "\n";
		} else {
			if ($debug)
				print "      Function does not exist!\n";
		}
	}
	return $cinfo;
}

function update_get_cached_plugin_info () {
	$cinfo = array();
	$info = db_fetch_assoc("SELECT * from plugin_update_info");
	if (is_array($info)) {
		foreach($info as $inf) {
			$cinfo[$inf['plugin']]['version']=$inf['version'];
			$cinfo[$inf['plugin']]['changes']=$inf['changes'];
		}
	}
	return $cinfo;
}

function cacti_version () {
	global $config;
	return array( 'name' 	=> 'cacti',
			'version' 	=> $config['cacti_version'],
			'longname'	=> 'Cacti',
			'author'	=> 'Cacti Developers',
			'homepage'	=> 'http://cacti.net',
			'email'	=> '',
			'url'		=> 'http://cactiusers.org/cacti/versions.php'
			);
}

function cacti_plugin_arch_version () {
	global $plugin_architecture;
	return array( 'name' 	=> 'pluginarch',
			'version' 	=> $plugin_architecture['version'],
			'longname'	=> 'Plugin Architecture',
			'author'	=> 'Jimmy Conner',
			'homepage'	=> 'http://cactiusers.org',
			'email'	=> 'jimmy@sqmail.org',
			'url'		=> 'http://cactiusers.org/cacti/versions.php'
			);
}

?>