<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2007 The Cacti Group                                      |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/



function update_version () {
	return array( 	'name' 		=> 'update',
			'version' 	=> '0.4',
			'longname'	=> 'Update Checker',
			'author'	=> 'Jimmy Conner',
			'homepage'	=> 'http://cactiusers.org',
			'email'	=> 'jimmy@sqmail.org',
			'url'		=> 'http://cactiusers.org/cacti/versions.php'
			);
}

function plugin_init_update() {
	global $plugin_hooks;
	$plugin_hooks['config_arrays']['update'] = 'update_config_arrays';
	$plugin_hooks['draw_navigation_text']['update'] = 'update_draw_navigation_text';
	$plugin_hooks['poller_bottom']['update'] = 'update_poller_bottom';
	$plugin_hooks['config_settings']['update'] = 'update_config_settings';
	$plugin_hooks['console_after']['update'] = 'update_console_after';
}

function update_console_after () {
	global $config, $plugins;
	update_setup_table();
	if (api_user_realm_auth('update.php')) {
		include_once($config['base_path'] . '/plugins/update/functions.php');
		$x = 0;
		$cinfo = update_get_plugin_info ();
		$info = update_get_cached_plugin_info();

		foreach ($plugins as $plugin) {
			if (isset($cinfo[$plugin]) && isset($info[$plugin]) && update_check_if_newer($cinfo[$plugin]['version'], $info[$plugin]['version']))
				$x++;
		}
		if ($x > 0) {
			print "<table width='98%' align='center'>	<tr><td class='textArea'>";
			print "<br><br><strong>$x Plugin" . ($x > 1 ? "s have" : " has") . " new <a href='" . $config['url_path'] . "plugins/update/update.php?tab=updates'>Updates</a> to install!</strong>";
			print "</td></tr></table>";
		}
	}
}

function update_config_settings () {
	global $tabs, $settings;
	$tabs["misc"] = "Misc";

	$temp = array(
		"update_header" => array(
			"friendly_name" => "Plugin Updates",
			"method" => "spacer",
			),
		"plugin_update_check_time" => array(
			"friendly_name" => "Update Scan Interval",
			"description" => "The amount of time in between checking for Updates.",
			"method" => "drop_array",
			"array" => array(0 => 'Never', 1 => '1 Day', 2 => '2 Days', 3 => '1 Week', 4 => '1 Month'),
			)
	);
	if (isset($settings["misc"]))
		$settings["misc"] = array_merge($settings["misc"], $temp);
	else
		$settings["misc"]=$temp;
}

function update_poller_bottom () {
	global $config;
	update_setup_table();
	$interval = read_config_option("plugin_update_check_time");
	if ($interval == '' || $interval > 4 || $interval < 0)
		$interval = 0;
	if ($interval == 0)
		return;

	$s = array(86400, 172800, 604800, 2678400);
	$seconds = $s[$interval - 1];
	$last_check = read_config_option("plugin_update_last_check");
	if ($last_check < 1)
		$last_check = 0;

	$current_time = time();
	if ($current_time < $last_check + $seconds)
		return;

	$command_string = read_config_option("path_php_binary");
	$extra_args = "-q " . $config["base_path"] . "/plugins/update/check.php";
	exec_background($command_string, "$extra_args");
}

function update_config_arrays () {
	global $user_auth_realms, $user_auth_realm_filenames, $menu;

	$user_auth_realms[47]='Check for Updates';
	$user_auth_realm_filenames['update.php'] = 47;
	$temp = $menu["Utilities"]['logout.php'];
	unset($menu["Utilities"]['logout.php']);
	$menu["Utilities"]['plugins/update/update.php'] = "Updates";
	$menu["Utilities"]['logout.php'] = $temp;
}

function update_draw_navigation_text ($nav) {
	$nav["update.php:"] = array("title" => "Updates", "mapping" => "index.php:", "url" => "update.php", "level" => "1");
	return $nav;
}

function update_setup_table () {
	global $config, $database_default;

	include_once($config["library_path"] . "/adodb/adodb.inc.php");
	include_once($config["library_path"] . "/database.php");
	$sql = "show tables from `" . $database_default . "`";

	$result = db_fetch_assoc($sql) or die (mysql_error());

	$tables = array();
	$sql = array();

	foreach($result as $index => $arr) {
		foreach ($arr as $t) {
			$tables[] = $t;
		}
	}

	if (!in_array('plugin_update_info', $tables)) {
		$sql[] = "CREATE TABLE plugin_update_info (
				plugin varchar(32) default NULL,
				name varchar(32) default NULL,
				version varchar(24) default NULL,
				author varchar(64) default NULL,
				homepage varchar(64) default NULL,
				email varchar(64) default NULL,
				released varchar(18) default NULL,
				changes text NOT NULL default ''
				)TYPE=MyISAM;";
		$sql[] = "INSERT INTO `user_auth_realm` VALUES (47, 1);";
	}

	if (!empty($sql)) {
		for ($a = 0; $a < count($sql); $a++) {
			$result = mysql_query($sql[$a]);
		}
	}
}

?>